var gulp        = require('gulp');
var connect     = require('gulp-connect');
var uglify      = require('gulp-uglify');
var ejs         = require('gulp-ejs');
var sass        = require('gulp-sass');
var rename      = require('gulp-rename');
var del         = require('del');
var watch       = require('gulp-watch');
var gutil       = require('gulp-util');
var express     = require('express');
var fs          = require('fs');
var mkdirp      = require('mkdirp');
var merge       = require('merge-stream');
var browserify  = require('browserify');
var source      = require('vinyl-source-stream');

var root        = __dirname + '/app';
var temp        = __dirname + '/tmp';
var namespaces  = [];

function uncache(module) {
  delete require.cache[require.resolve(module)];
}

function requireUncached(module){
  uncache(module);
  return require(module);
}

gulp.task('clean', function(cb) {
  del([temp + '/**'], cb);
});

gulp.task('namespaces', ['clean'], function() {
  fs.readdirSync(root).forEach(function(item) {
    if (fs.statSync([root, item].join('/')).isDirectory() && item !== 'shared') {
      namespaces.push(item);
    }
  });  
});

gulp.task('shared', ['namespaces'], function() {
  var shared = gulp.src([root, 'shared', '**'].join('/'));
  namespaces.forEach(function(ns) {
    shared = shared.pipe(gulp.dest([temp, '/', ns].join('/')));
  });
  return shared;
});

gulp.task('specific', ['shared'], function() {
  var streams = namespaces.map(function(ns) {
    return gulp.src([root, ns, '**'].join('/'))
      .pipe(gulp.dest([temp, ns].join('/')));
  });
  
  return merge.apply(merge, streams).on('error', function(err) { console.log(err); });
});

gulp.task('require', ['specific'], function() {
  
  var streams = namespaces.map(function(ns) {
    
    var list = function(path) {
      var str = [temp, ns, path].join('/');
      if (!fs.existsSync(str)) return [];
      return fs.readdirSync(str).map(function(ctl) {
        return ctl;
      });
    }
    
    var views = [];
    
    list('views').filter(function(item) {
      return ['components', 'directives', 'filters'].indexOf(item) === -1;
    }).forEach(function(item) {
      if (fs.statSync([temp, ns, 'views', item].join('/')).isDirectory()) {
        list('views/' + item).forEach(function(view) {
          views.push(item + '/' + view);
        });
      } else {
        views.push(item);
      }
    }); // some views are namespaces
    
    
    var namespace = {
      namespace: ns,
      controllers: list('controllers').map(function(item){ return item.split('.').shift() }),
      observers: list('observers').map(function(item) { return item.split('.').shift() }),
      views: {
        components: list('views/components'),
        directives: list('views/directives'),
        filters: list('views/filters'),
        views: views
      }
    };
    
    
    return gulp.src(require.resolve('vue-engine/bundle.ejs'))
      .pipe(ejs(namespace, { ext: '.js' }))
      .on('error', gutil.log)
      .pipe(gulp.dest([temp, ns].join('/')));
  });
  
  if (streams.length > 1) {
    return merge.apply(merge, streams).on('error', function(err) { console.log(err); });;
  } else {
    return streams[0];
  }
});


gulp.task('build', ['require'], function() {
  var streams = namespaces.map(function(ns) {
    
    var bundler = browserify([temp, ns, '/init.js'].join('/'));
    
    // bundler.transform({ global: true }, 'uglifyify');
    
    var bundle = bundler.bundle();
    
    return bundle
      .pipe(source('script.js'))
      .pipe(gulp.dest('./public/' + ns));
  });

  namespaces.forEach(function(ns) {
    streams.push(
      gulp.src([temp, ns, 'init.scss'].join('/'))
      .pipe(sass())
      .pipe(rename('style.css'))
      .pipe(gulp.dest('./public/' + ns))
      .pipe(connect.reload()));
  })
  
  namespaces.forEach(function(ns) {
    streams.push(gulp.src([temp, ns, 'index.html'].join('/')).pipe(gulp.dest('./public/' + ns)).pipe(connect.reload()));
  });
  
  return merge.apply(merge, streams);
});

gulp.task('watch', ['build'], function() {
  watch({ glob: root + '/**' }, function() {
    gulp.run('build');
  });
  
  namespaces.forEach(function(ns) {
    var services;
    var p = [root, ns, 'services'].join('/');
    
    try {
      services = fs.readdirSync(p).map(function(item) {
        return p + '/' + item.split('.')[0];
      });  
    } catch (err) {
      services = [];
      console.log(ns, 'doe not contain services');
    }
    
    watch({ glob: [root, ns, 'services', '**'].join('/') }, function() {
      services.forEach(uncache);
      connect.serverClose();
      gulp.run('connect');
    });
  });
  
});

gulp.task('connect', function() {
  return connect.server({
    root: './public',
    port: 3000,
    livereload: true,
    plugins: [requireUncached('./server')] 
  });
});

gulp.task('server', ['connect', 'watch']);

gulp.task('default', ['build']);

